/*
 * SFO Reader
 * Copyright (C) 2011 Matthieu Milan
 */

#include "sfo.h"

void SFO::work(char *sfo, char *entry) {
	char *basenamec = strdup(sfo);
	char *dirnamec = strdup(sfo);

	char *baseName = basename(basenamec);
	char *dirName = dirname(dirnamec);

	File f;
	if (f.open(baseName, dirName)) {
		if (f.getInt(0) != SFO_MAGIC) {
			printf("This file is not a PS3 SFO\n");
			exit(EXIT_FAILURE);
		}

		uint32_t numLines = f.getInt(16);
		for (uint32_t i = 0; i < numLines; i++) {
			char buffer[1024];
			char buffer2[1024];

			uint32_t key  = f.getKey(buffer, i);
			uint32_t type = f.getDataType(i);

			char *out = NULL;
			uint32_t outSize = 0;
			switch (type) {
				case TEXT:
					{
						uint32_t len = f.getValueText(buffer2, i);
						outSize = key + len + 6;
						out = (char *)malloc(outSize);
						snprintf(out, outSize, "T: %s = %s", buffer, buffer2);
					}
					break;

				case NUM:
					{
						sprintf(buffer2, "%d", f.getValueInt(i));
						outSize = key + strlen(buffer2) + 7;
						out = (char *)malloc(outSize);
						snprintf(out, outSize, "N: %s = %s", buffer, buffer2);
					}
					break;

				case BIN:
					{
						uint32_t len = f.getValueBin(buffer2, i);
						outSize = key + len + 6;
						out = (char *)malloc(outSize);
						snprintf(out, outSize, "B: %s = %s", buffer, buffer2);
					}
					break;

			}

			if (strcmp(entry, "") == 0)
				printf("%s\n", out);
			else if (strcmp(entry, buffer) == 0)
				printf("%s\n", buffer2);

			if (out != NULL)
				free(out);
		}

		f.close();
	}

	free(dirnamec);
	free(basenamec);
}
